import machine
import ubinascii
import time
from growmax.utils import sensors

import config

headers = {'content-type': 'application/json'}


def get_device_metadata():
    report_data = {}
    try:
        time.sleep(1.0)
        device_id = ubinascii.hexlify(machine.unique_id()).decode()
        time.sleep(1.0)
        report_data["device_metadata"] = {
            "device_id": device_id,
            "name": config.DEVICE_NAME,
        }
    except Exception as e:
        print(e)
    return report_data


def add_adafruit_scd4x_data_to_report(report_data, temp, rh, ppm_carbon_dioxide):
    if temp:
        report_data["temp"] = {
            "temp": temp,
            "unit": "C"
        }
    if rh:
        report_data["rh"] = {
            "rh": rh,
        }
    if ppm_carbon_dioxide:
        report_data["co2"] = {
            "ppm": ppm_carbon_dioxide,
        }


def report_environment_data(report_data):
    """This method requires installing urequests and ujson from pypi."""
    try:
        import urequests
        import ujson
        time.sleep(1.0)
        resp = urequests.post(
            "https://api.opensensor.io/environment/",
            headers=headers,
            data=ujson.dumps(report_data))
        print(resp.status_code)
        resp.close()
    except Exception as e:
        print(e)



import utime

def statistically_has_water(water_sensor):
    for x in range(0, 3):
        water_in_bucket = not water_sensor.value()
        if not water_in_bucket:
            return False
        utime.sleep(0.5)
    return True



import gc
import random
import utime
from machine import Pin

from growmax.atlas_ph.i2c import AtlasPHI2C
from growmax.moisture import Moisture
from growmax.pump import Pump
from growmax.utils import api
from growmax.utils.configs import get_moisture_threshold_for_position
from growmax.utils.displays import boot_sequence, display_basic_stats, display_ph_reading, display_scd4x_reading
from growmax.utils.mcu import get_gpio_for_mcu
from growmax.utils.sensors import init_adafruit_scd4x, read_adafruit_scd4x
from growmax.utils.water import statistically_has_water
from growmax.utils.wifi import ensure_wifi_connected

# User's config file
import config

# set the random seed, so messages are randomized
random.seed()


def main():
    boot_sequence()

    water_sensor = None
    if config.WATER_SENSOR_LOW_ENABLED:
        water_sensor = Pin(get_gpio_for_mcu(config.WATER_SENSOR_LOW), Pin.IN, Pin.PULL_DOWN)
    scd40x = None
    atlas_ph = None

    soil_sensors = [Moisture(channel=1), Moisture(channel=2), Moisture(channel=3), Moisture(channel=4),
                    Moisture(channel=5), Moisture(channel=6), Moisture(channel=7), Moisture(channel=8)]
    pumps = [Pump(channel=1), Pump(channel=2), Pump(channel=3), Pump(channel=4),
             Pump(channel=5), Pump(channel=6), Pump(channel=7), Pump(channel=8)]

    while True:
        utime.sleep(5.0)
        try:
            ensure_wifi_connected()
            utime.sleep(0.5)
        except Exception:
            # Potentially no wi-fi
            pass

        if config.ADAFRUIT_SCD4X_ENABLED and scd40x is None:
            scd40x = init_adafruit_scd4x(config.ADAFRUIT_SCD4X_I2C_CHANNEL)

        if hasattr(config, "ATLAS_PH_METER_ENABLED") and config.ATLAS_PH_METER_ENABLED:
            try:
                atlas_ph = AtlasPHI2C(config.ATLAS_PH_I2C_CHANNEL)
            except Exception as e:
                print(f"Error initializing Atlas pH probe: {e}")

        soil_moistures = []
        for position, soil_sensor in enumerate(soil_sensors):
            try:
                pump_position = str(position + 1)
                soil_moisture = soil_sensor.moisture
                soil_moistures.append(soil_moisture)
                has_water = water_sensor and statistically_has_water(water_sensor)
                moisture_config = get_moisture_threshold_for_position(position)
                print("Position ", pump_position,
                      " reservoir has water ", has_water,
                      " and moisture value ", soil_moisture, "/", moisture_config)
                display_basic_stats(has_water, pump_position, soil_moisture, moisture_config)
                if (config.PUMP_WHEN_DRY or has_water) and soil_moisture >= moisture_config:
                    print("position: ", pump_position)
                    pumps[position].dose(1, config.PUMP_CYCLE_DURATION)
                utime.sleep(2)
            except Exception as e:
                print("Exception: ", str(e))

        ph_reading = None
        temp, rh, ppm_carbon_dioxide = None, None, None
        if atlas_ph:
            utime.sleep(1.0)
            ph_reading = atlas_ph.obtain_ph_reading()
        if scd40x:
            temp, rh, ppm_carbon_dioxide = read_adafruit_scd4x(scd40x)
        if config.OPEN_SENSOR_COLLECT_DATA:
            report_data = api.get_device_metadata()
            if scd40x:
                api.add_adafruit_scd4x_data_to_report(report_data, temp, rh, ppm_carbon_dioxide)
            report_data["moisture"] = {
                "readings": soil_moistures
            }
            if atlas_ph and ph_reading:
                report_data["pH"] = {
                    "pH": ph_reading
                }
            api.report_environment_data(report_data)
        if scd40x:
            display_scd4x_reading(temp, rh, ppm_carbon_dioxide)
            utime.sleep(3)
        if atlas_ph:
            display_ph_reading(ph_reading)
            utime.sleep(3)

        print("Completed iteration; soil_moisture's = ", str(soil_moistures))
        print("Free mem before garbage collection: ", str(gc.mem_free()))
        gc.collect()
        print("Free mem after garbage collection: ", str(gc.mem_free()))



import machine
import utime
from growmax.utils.mcu import i2c_channel_pins

import config


DEFAULT_ADDRESS = 0x63


class AtlasPHI2C:

    def __init__(self, i2c_channel=0):
        pin_scl, pin_sda = i2c_channel_pins(i2c_channel)
        i2c_channel_pins(i2c_channel)
        self.i2c_bus = machine.I2C(i2c_channel, scl=machine.Pin(pin_scl), sda=machine.Pin(pin_sda), freq=100000)

        if hasattr(config, "ATLAS_PH_METER_ADDRESS"):
            self.address = config.ATLAS_PH_METER_ADDRESS or DEFAULT_ADDRESS
        else:
            self.address = DEFAULT_ADDRESS

    def obtain_ph_reading(self):
        self.i2c_bus.writeto(self.address, 'R,\r'.encode())
        utime.sleep(1.0)
        data = self.i2c_bus.readfrom(self.address, 31)
        if data[0] == 1:  # Successful reading
            reading = data[1:].decode().strip('\x00')
        else:
            reading = None
            print("Unable to get reading--check Atlas Scientific pH device settings")
        return reading




import config

from growmax import constants


def get_gpio_for_mcu(rp2040_gpio):
    if getattr(config, "GROWMAX_MCU", None) and config.GROWMAX_MCU == "ESP32S3_BPI":
        return constants.RP2040_MCU_MAPPINGS["ESP32S3_BPI"][rp2040_gpio]
    return rp2040_gpio

def i2c_channel_pins(i2c_channel=0):
    pin_scl = 1
    pin_sda = 0
    if i2c_channel == 1:
        pin_scl = 19
        pin_sda = 18
    return get_gpio_for_mcu(pin_scl), get_gpio_for_mcu(pin_sda)
